
# This file will be included in the packed file generated by t/line.t

# Check that the name is the one of the packed file
is __FILE__, $main_file, '__FILE__';
# Check that the line is the one where the module code starts in the packed file
is __LINE__, 14, '__LINE__';

1;
